
=head1 NAME

SuikaWiki::Name - SuikaWiki WikiName Implementation

=head1 DESCRIPTION

C<SuikaWiki::Name> implements WikiName related functions used in SuikaWiki.
SuikaWiki cope with two styles of WikiName: internal (array reference) form
and external (string) form.  This module provides "serializer" and "parser"
converting to each form, as well as "resolver" getting absolute WikiName
from relative WikiName and base WikiName.

This module is part of SuikaWiki.

=cut

package SuikaWiki::Name;
use strict;
our $VERSION = do{my @r=(q$Revision: 1.4 $=~/\d+/g);sprintf "%d."."%02d" x $#r,@r};

use overload
    'eq' => sub {
      my ($n1, $n2) = @_;
      return 0 if not (ref $n1) or not (ref $n2);
      return 0 if @$n1 != @$n2;
      for (0..$#$n1) {
        return 0 if $n1->[$_] ne $n2->[$_];
      }
      return 1;
    },
    fallback => 1;

=head1 METHODS

=over 4

=item $name = SuikaWiki::Name->new ($name, %option)

Constructs a new instance of WikiName.  C<$name> can be either in external
or internal form and either in relative or absolute form.
C<wiki> or C<delimiter> option required when C<$name> is in external form.

Ensure <$name> is interpretable as an array reference when it is a reference
to something.

=cut

sub new ($;%) {
  my $class = shift;
  my ($name, %opt) = @_;
  my $self;
  if (ref $name) {
    $self = bless [@$name], $class;
  } else {
    my $root = $opt{root} || $opt{wiki}->{config}->{name}->{space}->{root};
    if ($name eq $root) {
      $self = bless [], $class;
    } else {
      my $delim = $opt{delimiter_reg} || $opt{delimiter}
               || $opt{wiki}->{config}->{name}->{space}->{separator_reg};
      $self = bless [split $delim, $name], $class;
    }
  }
  $self;
}

=item $string = $name->stringify (%option)

Returns WikiName in external (string) form.
C<delimiter> or C<wiki> option is required.

=cut

sub stringify ($%) {
  my ($self, %opt) = @_;
  if (@$self == 0) {
    $opt{root} || $opt{wiki}->{config}->{name}->{space}->{root};
  } else {
    my $delim = $opt{delimiter}
             || $opt{wiki}->{config}->{name}->{space}->{separator};
    join $delim, @$self;
  }
}

=item $name->append_component ($component)

Appends WikiName component.  C<$component> is either a reference
interpretable as an array reference or a string.

=cut

sub append_component ($$%) {
  my ($self, $component) = @_;
  push @$self, ref $component ? @$component : $component;
}

=item $name->prepend_component ($component)

Prepends WikiName component.  C<$component> is either a reference
interpretable as an array reference or a string.

=cut

sub prepend_component ($$%) {
  my ($self, $component) = @_;
  unshift @$self, ref $component ? @$component : $component;
}

=item $abs = $name->absolute (base => $base, %option)

Resolves WikiName to absolute form.  New instance is constructed.
Either C<self> and C<parent> options or C<wiki> option required
to specify "self" and "parent" indicator.

=cut

sub absolute ($%) {
  my ($self, %opt) = @_;
  my @abs = @{$opt{base} ||= []};
  my @rel = @$self;
  my $Self = $opt{self} || $opt{wiki}->{config}->{name}->{space}->{self};
  my $Parent = $opt{parent} || $opt{wiki}->{config}->{name}->{space}->{parent};
  while ($rel[0] eq $Self or $rel[0] eq $Parent) {
    if ($rel[0] eq $Parent) {
      $#abs-- if @abs;
    }
    shift @rel;
  }
  if (@rel == @$self) {
    if ($opt{nearest}) {
      my @base = @abs;
      SEEK: {
        last SEEK unless $opt{wiki}->{db};
        while (@base) {
          if ($opt{wiki}->{db}->exist ($opt{nearest}, [@base, @rel])) {
            @abs = (@base, @rel);
            last SEEK unless @{$opt{base}} == @abs and
                             join ($,, @{$opt{base}}) eq join ($,, @abs);
          }
          pop @base;
        }
        @abs = @rel;
      }
    } else {
      @abs = @rel;
    }
  } else {
    push @abs, @rel;
  }
  ref ($self)->new (\@abs);
}

=item $clone = $name->clone

Generates a clone of C<$name> object.

=cut

sub clone ($;%) {
  my $self = shift;
  bless [@$self], ref $self;
}

=head1 TO DO

More study needed to enable to include delimiter string in WikiName 
(string form) as part of data.

=head1 LICENCE

Copyright 2004 Wakaba <w@suika.fam.cx>.  All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

1; # $Date: 2004/04/26 00:54:14 $
